
// If there is not a mask for everything, then we define it now
if ($EverythingMask $= "")
   $EverythingMask = "";

// Globals to control the center and radius of the targeted zones
// No artillery fire will land outside this zone.
$Team1ArtilleryCenter = "433.18 -360.44 2000";
$Team2ArtilleryCenter = "-436.29 492.934 2000";
$ArtillerySpread = 75;

// Globals to control the walking fire
$Team1ArtilleryTarget = $Team1ArtilleryCenter;
$Team2ArtilleryTarget = $Team2ArtilleryCenter;

// Globals to control the Special Effects
$Team1Barrel1Position = "14.42 54.63 575.34";
$Team1Barrel2Position = "14.35 53.95 575.06";
$Team1Barrel3Position = "14.69 54.32 575.07";
$Team1Barrel4Position = "14.05 54.26 575.35";
$Team1Discharge1Position = "9.96 58.98 567.96";
$Team1Discharge2Position = "9.61 58.62 567.46";
$Team1Discharge3Position = "9.96 58.98 567.46";
$Team1Discharge4Position = "9.61 58.62 567.96";
$Team1FireVector = "0.288 -0.473 0.833";

$Team2Barrel1Position = "-14.87 82.97 575.34";
$Team2Barrel2Position = "-14.79 83.63 575.08";
$Team2Barrel3Position = "-15.15 83.30 575.07";
$Team2Barrel4Position = "-14.50 83.30 575.35";
$Team2Discharge1Position = "-10.51 78.50 567.97";
$Team2Discharge2Position = "-10.15 78.86 567.47";
$Team2Discharge3Position = "-10.51 78.50 567.48";
$Team2Discharge4Position = "-10.15 78.86 567.97";
$Team2FireVector = "-0.3186 0.5513 0.7711";

// Function to update all clients with the code for
// artillery and the custom datablocks involved.
function Transmitall()
{
   %i = 0;
   %Count = clientGroup.getCount();

   while (%i < %Count)
   {
      clientGroup.getObject(0).transmitDataBlocks(1);

      %i++;
   }
}

// Barge control program, as long as someone owns it it will fire at the
// other side.  To prevent altering a core function, we check every couple
// of seconds until it is captured.  Once it is captured, we have it fire
// the 4 barrels facing the other team just short of a half a second apart.
// The function is then scheduled to reload randomly within a range.
function ArtilleryBarrage()
{
   // If the barge no longer exists, we stop trying to fire
   if (isObject($ArtilleryBarge) == false)
      return;

   // If no one owns the barge, then we check again a little later until
   // someone does take it.
   if ($ArtilleryBarge.team == 0)
   {
      schedule(3500, 0, ArtilleryBarrage);
      return;
   }

   // Fire the barrels aimed at the other team's base
   if ($ArtilleryBarge.team == 1)
   {
      ArtilleryCannonFire($Team1Barrel1Position, $Team1FireVector, $Team1Discharge1Position);
      schedule(450, 0, ArtilleryCannonFire, $Team1Barrel2Position, $Team1FireVector, $Team1Discharge2Position);
      schedule(900, 0, ArtilleryCannonFire, $Team1Barrel3Position, $Team1FireVector, $Team1Discharge3Position);
      schedule(1350, 0, ArtilleryCannonFire, $Team1Barrel4Position, $Team1FireVector, $Team1Discharge4Position);

      // Schedule the sound of the turret adjusting slightly
      schedule(13000 + %Random, 0, serverPlay3D, DefaultDeploySound, $Team1Discharge4Position);
   } else {
      ArtilleryCannonFire($Team2Barrel1Position, $Team2FireVector, $Team2Discharge1Position);
      schedule(450, 0, ArtilleryCannonFire, $Team2Barrel2Position, $Team2FireVector, $Team2Discharge2Position);
      schedule(900, 0, ArtilleryCannonFire, $Team2Barrel3Position, $Team2FireVector, $Team2Discharge3Position);
      schedule(1350, 0, ArtilleryCannonFire, $Team2Barrel4Position, $Team2FireVector, $Team2Discharge4Position);

      // Schedule the sound of the turret adjusting slightly
      schedule(13000 + %Random, 0, serverPlay3D, DefaultDeploySound, $Team2Discharge4Position);
   }

   // Reschedule the next firing to between 16 and 22 seconds from now.
   %Random = getRandom() * 6000;   
   schedule(16000 + %Random, 0, ArtilleryBarrage);
}

// This function handles the firing of a barrel.
function ArtilleryCannonFire(%Position, %Vector, %DischargePosition)
{
   // Make the discharge smoke that comes out the back
   %DischargeSmoke = new Item()
   {
      datablock = ArtilleryDischargeSmoke;
      position = %DischargePosition;
   };
   MissionCleanUp.Add(%DischargeSmoke);
   schedule(450, 0, detonateGrenade, %DischargeSmoke);

   // Make the firing smoke that comes out the barrel
   %FiringSmoke = new Item()
   {
      datablock = ArtilleryBarrelSmoke;
      position = %Position;
   };
   MissionCleanUp.Add(%FiringSmoke);
   schedule(550, 0, detonateGrenade, %FiringSmoke);

   // schedule the spawning of the shell.
   schedule(700, 0, ArtilleryShellFire, %Position, %Vector);

   // We schedule the reloading sound now.
   schedule(15000, 0, ArtilleryReload, %DischargePosition);
}

// This function fires mortar from the barrels and then schedules thier
// removal and replacement with artillary.  Since people can stand on the
// cannon and might be hit as it fires, we do a check now for people in
// the way and then fire a linear shell if someone is (since it has no 
// timing delay).
function ArtilleryShellFire(%Position, %Vector)
{
   echo("Firing Cannon");

   %EndPosition = VectorAdd(%Position, VectorScale(%Vector, 200));

   %TargetObject = ContainerRayCast(%Position, %EndPosition, $EverythingMask);

   if (%TargetObject !$= "0")
   {
      %Data = LinearArtilleryShot;
      %ArtilleryShell = new LinearProjectile()
      {
         dataBlock        = %Data;
         initialDirection = %Vector;
         initialPosition  = %Position;
         scale            = %Data.scale;
         team             = $ArtilleryBarge.team;
      };
   } else {
      %Data = ArtilleryShot;
      %ArtilleryShell = new GrenadeProjectile()
      {
         dataBlock        = %Data;
         initialDirection = %Vector;
         initialPosition  = %Position;
         scale            = %Data.scale;
         team             = $ArtilleryBarge.team;
      };
   }
   MissionCleanup.add(%ArtilleryShell);

   echo("Shell: " @ %ArtilleryShell);

   serverPlay3D(ArtilleryFireSound, %Position);

   // Once the shell is out of sight, start the fading process
   %ArtilleryShell.schedule(850, "setCloaked", true);

   // We schedule the Replacement check now.
   schedule(1500, 0, ArtilleryShotReplacement, %ArtilleryShell);
}

// This function plays the sound of the barrel reloading.
function ArtilleryReload(%Position)
{
   serverPlay3D(ArtilleryReloadSound, %Position);
}

function ArtilleryShotReplacement(%ArtilleryShot)
{
   // If something happened to the shell (most likely it hit something)
   // then don't replace it with an artillery shell.
   if (isObject(%ArtilleryShot) == false)
      return;

   echo("Artillery Replacement");

   if (%ArtilleryShot.team == 1)
   {
      %Position = $Team1ArtilleryTarget;
      %AbsolutePosition = $Team1ArtilleryCenter;
   } else {
      %Position = $Team2ArtilleryTarget;
      %AbsolutePosition = $Team2ArtilleryCenter;
   }

   %PositionX = getWord(%Position, 0);
   %PositionY = getWord(%Position, 1);

   // Attempt to pick a new random location based on where the last one landed
   // Compare the location to the absolute one passed and if within the spread
   // accept it and prepare for the next firing.
   %PositionFound = false;
   while (%PositionFound == false)
   {
      %PositionOffsetX = (getRandom() * $ArtillerySpread * 0.85) - ($ArtillerySpread * 0.45);
      %PositionOffsetY = (getRandom() * $ArtillerySpread * 0.85) - ($ArtillerySpread * 0.45);

      %TestPosition = %PositionX + %PositionOffsetX @ " " @ %PositionY + %PositionOffsetY @ " 2000";

      %DistanceFromCenter = VectorLen(VectorSub(%AbsolutePosition, %TestPosition));

      if (%DistanceFromCenter <= $ArtillerySpread)
         %PositionFound = true;
   }

   // Store the location as a global for the next shot to land near

   if (%ArtilleryShot.team == 1)
   {
      $Team1ArtilleryTarget = %TestPosition;
   } else {
      $Team2ArtilleryTarget = %TestPosition;
   }

   %ArtilleryShot.delete();

   // Create a new shell and throw it towards the ground
   %ArtilleryShell = new Item()
   {
      dataBlock = ArtilleryShell;
      position = %TestPosition;
      scale = ArtilleryShell.Scale;
      team = %ArtilleryShot.team;
   };
   MissionCleanUp.Add(%ArtilleryShell);

   %ArtilleryShell.ApplyImpulse(%Position, "0 0 -100");

   %ArtilleryShell.schedule(1500, playAudio, 0, BomberBombProjectileSound);
}

datablock ExplosionData(ArtillerySubExplosion)
{
   debris = SatchelDebris;
   debrisThetaMin = 10;
   debrisThetaMax = 170;
   debrisNum = 16;
   debrisVelocity = 60.0;
   debrisVelocityVariance = 15.0;

   lifetimeMS = 2000;
   delayMS = 100;

   emitter[0] = SatchelExplosionSmokeEmitter;
   emitter[1] = SatchelSparksEmitter;

   offset = 9.0;

   sizes[0] = "1.0 1.0 1.0";
   sizes[1] = "1.0 1.0 1.0";
   times[0] = 0.0;
   times[1] = 1.0;
};

datablock ExplosionData(ArtilleryExplosion)
{
   soundProfile = SatchelChargePreExplosionSound;
   
   subExplosion[0] = ArtillerySubExplosion;
   subExplosion[1] = ArtillerySubExplosion;
};

datablock LinearProjectileData(LinearArtilleryShot)
{
   projectileShapeName = "grenade_projectile.dts";
   scale     = "3.0 3.0 3.0";

   directDamage        = 0.0;
   hasDamageRadius     = true;
   indirectDamage      = 1.50;
   damageRadius        = 24;
   radiusDamageType    = $DamageType::Explosive;
   kickBackStrength    = 4500;

   explosion           = ArtilleryExplosion;
   velInheritFactor    = 0;

   dryVelocity         = 250;
   fizzleTimeMS        = 4000;
   lifetimeMS          = 5000;
   explodeOnDeath      = false;

   sound               = MortarProjectileSound;
};

datablock GrenadeProjectileData(ArtilleryShot)
{
   projectileShapeName = "grenade_projectile.dts";
   scale     = "3.0 3.0 3.0";

   directDamage        = 0.0;
   hasDamageRadius     = true;
   indirectDamage      = 1.50;
   damageRadius        = 24;
   radiusDamageType    = $DamageType::Explosive;
   kickBackStrength    = 4500;

   explosion           = ArtilleryExplosion;
   velInheritFactor    = 0;

   grenadeElasticity   = 0.0;
   grenadeFriction     = 1.0;
   armingDelayMS       = 250;
   muzzleVelocity      = 250;
   drag                = 0.0;

   sound               = MortarProjectileSound;
};

datablock ItemData(ArtilleryBarrelSmoke)
{
   className = Other;
   shapeFile = "turret_muzzlepoint.dts";

   mass = 10.0;

   sticky = true;

   elasticity = 0.0;
   friction = 1.0;

   maxDamage = 100;

   explosion = HandGrenadeExplosion;

   indirectDamage   = 0.4;
   damageRadius     = 10.0;
   radiusDamageType = $DamageType::Explosion;
   kickBackStrength = 2000;
};

datablock ItemData(ArtilleryDischargeSmoke)
{
   className = Other;
   shapeFile = "turret_muzzlepoint.dts";

   mass = 0.1;

   sticky = true;

   elasticity = 0.0;
   friction = 1.0;

   maxDamage = 100;

   explosion = DeployablesExplosion;

   indirectDamage   = 0.1;
   damageRadius     = 2.0;
   radiusDamageType = $DamageType::Explosion;
   kickBackStrength = 800;
};

datablock AudioDescription(LouderAudioClose3d)
{
   volume   = 3.0;
   isLooping= false;

   is3D     = true;
   minDistance= 10.0;
   MaxDistance= 90.0;
   type     = $EffectAudioType;
};

datablock AudioDescription(LoudAudioClosest3d)
{
   volume   = 1.6;
   isLooping= false;

   is3D     = true;
   minDistance= 5.0;
   MaxDistance= 20.0;
   type     = $EffectAudioType;
};

datablock AudioProfile(ArtilleryFireSound)
{
   filename    = "fx/weapons/mortar_explode.wav";
   description = LouderAudioClose3d;
   preload = true;
};

datablock AudioProfile(ArtilleryReloadSound)
{
   filename    = "fx/weapons/mortar_reload.wav";
   description = LoudAudioClosest3d;
   preload = true;
};


datablock ItemData(ArtilleryShell)
{
   shapeFile = "grenade_projectile.dts";
   scale     = "3.0 3.0 3.0";

   mass = 1.0;
   elasticity = 0.0;
   friction = 1;
   drag = 0.1;

   sticky = true;

   hasDamageRadius     = true;
   indirectDamage      = 1.5;
   damageRadius        = 26;
   radiusDamageType    = $DamageType::Explosion;
   kickBackStrength    = 3250;

   explosion           = "VehicleBombExplosion";

   sound                = BomberBombProjectileSound;

   emap = true;
};

// Let it explode if it hits an object.
function ArtilleryShell::onCollision(%Data, %Shell, %ColidingObject)
{
   %Shell.setDamageState(Destroyed);
}

// Let it explode if it hits the ground or some other non-object.
function ArtilleryShell::onStickyCollision(%Data, %Shell)
{
   %Shell.setDamageState(Destroyed);
}

// Let it explode if it goes into the water.
function ArtilleryShell::onEnterLiquid(%Data, %Shell, %Coverage, %Type)
{
   %Shell.setDamageState(Destroyed);
}

function ArtilleryShell::onDestroyed(%Data, %Shell, %prevState)
{
   // If it already exploded somehow, ignore the second attempt
   if (%Shell.detonated == true)
      return;

   %Position = posFromTransform(%Shell.getTransform());

   %Radius = %Data.damageRadius;
   %Damage = %Data.indirectDamage;
   %KickBack = %Data.kickBackStrength;
   %DamageType = %Data.radiusDamageType;

   // Explosion Damage
   RadiusExplosion(%Shell, %Position, %Radius, %Damage, %KickBack, 0, %DamageType);

   %Shell.detonated = true;
   %Shell.schedule(500,"delete");
}

transmitall();
